package betterwithmods.common.blocks;

import betterwithmods.common.BWMBlocks;
import betterwithmods.common.tile.TileLamp;
import betterwithmods.library.common.block.BlockBase;
import betterwithmods.library.common.block.IBlockActive;
import betterwithmods.library.utils.colors.ColorUtils;
import betterwithmods.util.WorldUtils;
import net.minecraft.block.Block;
import net.minecraft.block.SoundType;
import net.minecraft.block.material.Material;
import net.minecraft.block.properties.IProperty;
import net.minecraft.block.properties.PropertyBool;
import net.minecraft.block.state.BlockFaceShape;
import net.minecraft.block.state.BlockStateContainer;
import net.minecraft.block.state.IBlockState;
import net.minecraft.client.renderer.color.IBlockColor;
import net.minecraft.client.renderer.color.IItemColor;
import net.minecraft.creativetab.CreativeTabs;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.init.SoundEvents;
import net.minecraft.item.EnumDyeColor;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.*;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.math.RayTraceResult;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.util.Optional;

public class BlockLamp extends BlockBase implements IBlockActive, IHempGrower, IBlockColor, IItemColor {
    private static final PropertyBool INVERTED = PropertyBool.func_177716_a("inverted");

    public BlockLamp() {
        super(Material.field_151591_t);
        func_149711_c(5.0F);
        func_149752_b(10.0F);
        func_149672_a(SoundType.field_185853_f);
    }

    @Override
    public void func_149666_a(CreativeTabs itemIn, NonNullList<ItemStack> items) {
        for (EnumDyeColor color : ColorUtils.DYES) {
            items.add(createLamp(ColorUtils.getDyeColor(color)));
        }
    }

    @Nonnull
    @Override
    public ItemStack getPickBlock(@Nonnull IBlockState state, RayTraceResult target, @Nonnull World world, @Nonnull BlockPos pos, EntityPlayer player) {
        return getTile(world, pos).map(TileLamp::getItemStack).orElse(ItemStack.field_190927_a);
    }

    public static ItemStack createLamp(int color) {
        ItemStack stack = new ItemStack(BWMBlocks.LAMP);
        NBTTagCompound tag = new NBTTagCompound();
        tag.func_74768_a("color", color);
        stack.func_77982_d(tag);
        return stack;
    }

    @Override
    public IProperty<?>[] getProperties() {
        return new IProperty[]{ACTIVE, INVERTED};
    }

    @Override
    public int func_176201_c(IBlockState state) {

        int active = state.func_177229_b(ACTIVE) ? 1 : 0;
        int inverted = state.func_177229_b(INVERTED) ? 1 : 0;

        return active | inverted << 1;
    }

    @Override
    public IBlockState func_176203_a(int meta) {
        boolean active = (meta & 1) == 1;
        boolean inverted = (meta >> 1 & 1) == 1;
        return func_176223_P().func_177226_a(ACTIVE, active).func_177226_a(INVERTED, inverted);
    }

    @Override
    public boolean hasTileEntity(IBlockState state) {
        return true;
    }

    @Nullable
    @Override
    public TileEntity createTileEntity(World world, IBlockState state) {
        return new TileLamp();
    }

    @Override
    public boolean canGrowHemp(IBlockAccess world, BlockPos pos, IBlockState state) {
        return isActive(state);
    }

    @Override
    public int getLightValue(IBlockState state, IBlockAccess world, BlockPos pos) {
        boolean active = isActive(state);
        return active ? 15 : 0;
    }

    @SuppressWarnings("deprecation")
    @Override
    public boolean func_149686_d(IBlockState state) {
        return false;
    }

    @SuppressWarnings("deprecation")
    @Override
    public boolean func_149662_c(IBlockState state) {
        return false;
    }

    @Nonnull
    @SideOnly(Side.CLIENT)
    public BlockRenderLayer func_180664_k() {
        return BlockRenderLayer.TRANSLUCENT;
    }


    @SuppressWarnings("deprecation")
    @Override
    public boolean func_176225_a(@Nonnull IBlockState state, @Nonnull IBlockAccess access, @Nonnull BlockPos pos, EnumFacing side) {
        IBlockState offsetState = access.func_180495_p(pos.func_177972_a(side));
        if (offsetState != state)
            return true;
        if (offsetState.func_177230_c() == this)
            return false;
        return super.func_176225_a(state, access, pos, side);
    }

    @Override
    public void func_189540_a(IBlockState state, World worldIn, BlockPos pos, Block blockIn, BlockPos fromPos) {
        onChanged(worldIn, pos, state);
    }

    @Override
    public void func_176213_c(World worldIn, BlockPos pos, IBlockState state) {
        super.func_176213_c(worldIn, pos, state);
        onChanged(worldIn, pos, state);
    }

    public void onChanged(World world, BlockPos pos, IBlockState state) {
        if (!world.field_72995_K) {
            boolean active = isActive(state);
            boolean redstone = WorldUtils.isRedstonePowered(world, pos);
            if (active != redstone) {
                setActive(world, pos, redstone);
            }
        }
    }

    @Override
    public boolean recolorBlock(World world, @Nonnull BlockPos pos, EnumFacing side, @Nonnull EnumDyeColor color) {
        if (!world.field_72995_K) {
            getTile(world, pos).ifPresent(t -> t.setColor(ColorUtils.getDyeColor(color)));
            return true;
        }
        return false;
    }


    @Override
    public int func_186720_a(@Nonnull IBlockState state, @Nullable IBlockAccess world, @Nullable BlockPos pos, int tintIndex) {
        if (world != null && pos != null) {
            return getTile(world, pos).map(TileLamp::getColor).orElse(0);
        }
        return 0;
    }

    @Override
    public int func_186726_a(@Nonnull ItemStack stack, int tintIndex) {
        return getStackColor(stack);
    }

    public static int getStackColor(ItemStack stack) {
        NBTTagCompound tag = stack.func_77978_p();
        if (tag != null) {
            return tag.func_74762_e("color");
        }
        return 0;
    }

    public Optional<TileLamp> getTile(IBlockAccess world, BlockPos pos) {
        TileEntity tile = world.func_175625_s(pos);
        return tile instanceof TileLamp ? Optional.of((TileLamp) tile) : Optional.empty();
    }


    @Nonnull
    @Override
    public BlockFaceShape func_193383_a(IBlockAccess worldIn, IBlockState state, BlockPos pos, EnumFacing face) {
        return BlockFaceShape.SOLID;
    }

    @Override
    public boolean func_180639_a(World worldIn, BlockPos pos, IBlockState state, EntityPlayer playerIn, EnumHand hand, EnumFacing facing, float hitX, float hitY, float hitZ) {
        if (worldIn.field_72995_K)
            return true;
        if (playerIn.func_70093_af()) {
            if (hand == EnumHand.MAIN_HAND) {
                float f = state.func_177229_b(INVERTED) ? 0.6F : 0.5F;
                worldIn.func_184133_a(null, pos, SoundEvents.field_187750_dc, SoundCategory.BLOCKS, 0.3F, f);
                setInverted(worldIn, pos, state, !isInverted(state));
            }

        }
        return super.func_180639_a(worldIn, pos, state, playerIn, hand, facing, hitX, hitY, hitZ);
    }

    private boolean isInverted(IBlockState state) {
        return state.func_177229_b(INVERTED);
    }

    private void setInverted(World world, BlockPos pos, IBlockState state, boolean inverted) {
        world.func_175656_a(pos, state.func_177226_a(INVERTED, inverted));
    }

    @Override
    public boolean isActive(IBlockState state) {
        return IBlockActive.super.isActive(state);
    }
}

