package betterwithmods.common.items.itemblocks;

import betterwithmods.api.block.IRenderRotationPlacement;
import betterwithmods.common.BWMBlocks;
import betterwithmods.common.blocks.mechanical.BlockAxle;
import betterwithmods.common.blocks.mechanical.BlockAxleGenerator;
import betterwithmods.library.utils.DirUtils;
import betterwithmods.library.utils.LocaleUtils;
import betterwithmods.util.WorldUtils;
import net.minecraft.block.Block;
import net.minecraft.block.state.IBlockState;
import net.minecraft.client.renderer.GlStateManager;
import net.minecraft.client.renderer.RenderGlobal;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.ItemBlock;
import net.minecraft.item.ItemStack;
import net.minecraft.util.EnumActionResult;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.EnumHand;
import net.minecraft.util.math.AxisAlignedBB;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.math.RayTraceResult;
import net.minecraft.util.math.Vec3d;
import net.minecraft.world.World;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

import javax.annotation.Nonnull;
import java.util.Objects;

import static betterwithmods.client.ClientEventHandler.getPlayerPositionDelta;

public abstract class ItemAxleGenerator extends ItemBlock implements IRenderRotationPlacement {
    protected int radius;

    public ItemAxleGenerator(Block block) {
        super(block);

        this.func_77625_d(16);
    }

    public abstract boolean isAxis(EnumFacing.Axis axis);

    private void showErrorMessage(EntityPlayer player, Error error) {
        String block = Objects.requireNonNull(getRegistryName()).func_110624_b();
        player.func_145747_a(LocaleUtils.getMessageComponent(getRegistryName().func_110624_b(), error.format(block)));
    }

    public AxisAlignedBB getBounds(EnumFacing.Axis axis, int radius) {
        if (axis != null) {
            switch (axis) {
                case X:
                    return new AxisAlignedBB(1 / 16f, -radius, -radius, 15 / 16f, radius + 1, radius + 1);
                case Z:
                    return new AxisAlignedBB(-radius, -radius, 1 / 16f, radius + 1, radius + 1, 15 / 16f);
            }
        }
        return Block.field_185506_k;
    }

    private boolean containsOtherGenerator(World world, BlockPos center, EnumFacing.Axis axis, EntityPlayer player) {
        int d = radius * 2;
        AxisAlignedBB box = getBounds(axis, d);
        Iterable<BlockPos> positions = WorldUtils.getPosInBox(box.func_186670_a(center));
        for (BlockPos p : positions) {
            IBlockState state = world.func_180495_p(p);
            if (state.func_177230_c() instanceof BlockAxleGenerator) {
                showErrorMessage(player, Error.CONFLICT);
                return true;
            }
        }
        return false;
    }

    private boolean isValidArea(World world, EntityPlayer player, BlockPos pos, EnumFacing.Axis axis) {
        AxisAlignedBB box = getBounds(axis, radius);
        if (box == null)
            return false;
        Iterable<BlockPos> positions = WorldUtils.getPosInBox(box.func_186670_a(pos));

        for (BlockPos p : positions) {
            if (onAxis(pos, p, axis)) {
                IBlockState state = world.func_180495_p(p);
                if (state.func_177230_c() instanceof BlockAxle) {
                    continue;
                } else {
                    showErrorMessage(player, Error.SPACE);
                    return false;
                }
            }
            IBlockState state = world.func_180495_p(p);
            if (!state.func_185904_a().func_76222_j()) {
                return false;
            }
        }
        return !containsOtherGenerator(world, pos, axis, player);
    }

    private boolean onAxis(BlockPos base, BlockPos test, EnumFacing.Axis axis) {
        switch (axis) {
            case X:
                return base.func_177952_p() == test.func_177952_p() && base.func_177956_o() == test.func_177956_o();
            case Y:
                return base.func_177952_p() == test.func_177952_p() && base.func_177958_n() == test.func_177958_n();
            case Z:
                return base.func_177958_n() == test.func_177958_n() && base.func_177956_o() == test.func_177956_o();
            default:
                return false;
        }
    }

    @Nonnull
    @Override
    public EnumActionResult func_180614_a(EntityPlayer player, World worldIn, @Nonnull BlockPos pos, @Nonnull EnumHand hand, @Nonnull EnumFacing facing, float hitX, float hitY, float hitZ) {
        EnumFacing.Axis axis = getAxleAxis(worldIn, pos);
        if (axis != null) {
            if (isValidArea(worldIn, player, pos, axis)) {
                worldIn.func_175656_a(pos, ((BlockAxleGenerator) field_150939_a).getAxisState(axis));
                return EnumActionResult.SUCCESS;
            } else {
                showErrorMessage(player, Error.SPACE);
            }
        } else {
            showErrorMessage(player, Error.AXLE);
        }
        return EnumActionResult.PASS;
    }

    protected EnumFacing.Axis getAxleAxis(World world, BlockPos pos) {
        IBlockState state = world.func_180495_p(pos);
        if (state.func_177230_c() instanceof BlockAxle) {
            EnumFacing.Axis axis = state.func_177229_b(DirUtils.AXIS);
            return isAxis(axis) ? axis : null;
        }
        return null;
    }

    @Override
    public AxisAlignedBB getBounds(World world, BlockPos pos, EnumFacing facing, float flX, float flY, float flZ, ItemStack stack, EntityLivingBase placer) {
        return getBounds(getAxleAxis(world, pos), this.radius);
    }

    @Override
    public void render(World world, Block block, BlockPos pos, ItemStack stack, EntityPlayer player, EnumFacing side, RayTraceResult target, double partial) {
        IBlockState state = world.func_180495_p(pos);
        if (state.func_177230_c() == BWMBlocks.WOODEN_AXLE) {
            Vec3d vec = target.field_72307_f.func_72441_c(-target.func_178782_a().func_177958_n(), -target.func_178782_a().func_177956_o(), -target.func_178782_a().func_177952_p());
            float x = (float) vec.field_72450_a, y = (float) vec.field_72448_b, z = (float) vec.field_72449_c;
            GlStateManager.func_179147_l();
            GlStateManager.func_187428_a(GlStateManager.SourceFactor.SRC_ALPHA, GlStateManager.DestFactor.ONE_MINUS_SRC_ALPHA, GlStateManager.SourceFactor.ONE, GlStateManager.DestFactor.ZERO);
            GlStateManager.func_187441_d(4.0F);
            GlStateManager.func_179090_x();
            GlStateManager.func_179132_a(false);

            double[] deltas = getPlayerPositionDelta(player, partial);

            double dx = deltas[0];
            double dy = deltas[1];
            double dz = deltas[2];

            AxisAlignedBB box = this.getBounds(world, pos, side, x, y, z, stack, player);

            if (box != null) {
                renderModel(world, pos, pos.func_177958_n() - dx, pos.func_177956_o() - dy, pos.func_177952_p() - dz, partial);
                RenderGlobal.func_189697_a(box.func_186662_g(0.002D).func_186670_a(pos).func_72317_d(-dx, -dy, -dz), 0.0F, 0.0F, 0.0F, 0.4F);
            }

            GlStateManager.func_179132_a(true);
            GlStateManager.func_179098_w();
            GlStateManager.func_179084_k();
        }
    }

    @SideOnly(Side.CLIENT)
    public abstract void renderModel(World world, BlockPos pos, double x, double y, double z, double partial);


    @SideOnly(Side.CLIENT)
    public enum Error {
        CONFLICT("generator.conflict"),
        SPACE("%s.space"),
        AXLE("%s.axle");

        public final String key;

        Error(String key) {
            this.key = key;
        }

        public String format(String block) {
            return String.format(key, block);
        }
    }
}
