package betterwithmods.common.potion;

import betterwithmods.module.tweaks.NetherFortressSpawns;
import net.minecraft.block.state.IBlockState;
import net.minecraft.client.Minecraft;
import net.minecraft.entity.EntityLiving;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.entity.EnumCreatureType;
import net.minecraft.util.EnumParticleTypes;
import net.minecraft.util.math.AxisAlignedBB;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.math.MathHelper;
import net.minecraft.world.DimensionType;
import net.minecraft.world.EnumSkyBlock;
import net.minecraft.world.World;
import net.minecraft.world.WorldEntitySpawner;

public class PotionTruesight extends BWMPotion {
    private final static AxisAlignedBB TEST_BB = new AxisAlignedBB(0.6D / 2D, 0, 0.6D / 2D, 1D - 0.6D / 2D, 1D, 1D - 0.6D / 2D);
    public static boolean ignoreLayer = false;

    public PotionTruesight(String name, boolean b, int potionColor) {
        super(name, b, potionColor);
    }

    //Borrowed from MoreOverlays
    private static boolean canSpawnMobsHere(World world, BlockPos pos) {
        DimensionType type = world.field_73011_w.func_186058_p();
        if (type == DimensionType.OVERWORLD || type == DimensionType.THE_END) {
            if (world.func_175642_b(EnumSkyBlock.BLOCK, pos) >= 8)
                return false;
        } else if (type == DimensionType.NETHER) {
            if (!NetherFortressSpawns.hasNetherFortress(world, pos))
                return false;
        }

        if (world.func_180494_b(pos).func_76747_a(EnumCreatureType.MONSTER).isEmpty())
            return false;

        if (!WorldEntitySpawner.func_180267_a(EntityLiving.SpawnPlacementType.ON_GROUND, world, pos))
            return false;

        return checkCollision(pos, world);
    }

    private static boolean checkCollision(BlockPos pos, World world) {
        IBlockState block1 = world.func_180495_p(pos);

        if (block1.func_185915_l() || (!ignoreLayer && world.func_180495_p(pos.func_177984_a()).func_185915_l())) //Don't check because a check on normal Cubes will/should return false ( 99% collide ).
            return false;
        else if (world.func_175623_d(pos) && (ignoreLayer || world.func_175623_d(pos.func_177984_a())))  //Don't check because Air has no Collision Box
            return true;

        AxisAlignedBB bb = TEST_BB.func_72317_d(pos.func_177958_n(), pos.func_177956_o(), pos.func_177952_p());
        if (world.func_184144_a(null, bb).isEmpty() && !world.func_72953_d(bb)) {
            if (ignoreLayer)
                return true;
            else {
                AxisAlignedBB bb2 = bb.func_72317_d(0, 1, 0);
                return world.func_184144_a(null, bb2).isEmpty() && !world.func_72953_d(bb2);
            }
        }
        return false;
    }

    @Override
    public void tick(EntityLivingBase entity) {
        World world = entity.func_130014_f_();
        if (world.field_72995_K) {
            Minecraft mc = Minecraft.func_71410_x();
            if (entity != mc.field_71439_g)
                return;
            int var3 = mc.field_71474_y.field_74362_aa;
            if (!mc.func_147113_T()) {

                int entityX = MathHelper.func_76128_c(entity.field_70165_t);
                int var5 = MathHelper.func_76128_c(entity.field_70163_u);
                int var6 = MathHelper.func_76128_c(entity.field_70161_v);
                int radius = 10;
                for (int x = entityX - radius; x <= entityX + radius; ++x) {
                    for (int y = var5 - radius; y <= var5 + radius; ++y) {
                        for (int z = var6 - radius; z <= var6 + radius; ++z) {
                            if (canSpawnMobsHere(world, new BlockPos(x, y, z)) && (var3 == 0 || world.field_73012_v.nextInt(12) <= 2 - var3 << 1)) {
                                double i = (double) x + world.field_73012_v.nextDouble();
                                double j = (double) y + world.field_73012_v.nextDouble() * 0.25D;
                                double k = (double) z + world.field_73012_v.nextDouble();
                                world.func_175688_a(EnumParticleTypes.SPELL_MOB, i, j, k, 0, 0, 0);
                            }
                        }
                    }
                }
            }
        }

    }


}
