package betterwithmods.module.conversion.beacons;

import betterwithmods.library.utils.colors.ColorUtils;
import betterwithmods.library.utils.ingredient.blockstate.BlockStateIngredient;
import betterwithmods.util.WorldUtils;
import com.google.common.collect.Lists;
import com.google.common.collect.Sets;
import net.minecraft.advancements.CriteriaTriggers;
import net.minecraft.block.Block;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.entity.player.EntityPlayerMP;
import net.minecraft.init.Blocks;
import net.minecraft.item.ItemBlock;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTBase;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.nbt.NBTTagList;
import net.minecraft.nbt.NBTUtil;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.ITickable;
import net.minecraft.util.math.AxisAlignedBB;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.World;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

import java.util.Arrays;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;

/**
 * Created by primetoxinz on 7/17/17.
 */
public class TileBeacon extends net.minecraft.tileentity.TileEntityBeacon implements ITickable {

    private static final int DEFAULT_TICK_RATE = 120;

    private int currentLevel;
    private boolean active;
    private IBlockState type = Blocks.field_150350_a.func_176223_P();
    private BeaconEffect effect, prevEffect;
    private int tick;
    private List<BeamSegment> segments = Lists.newArrayList();

    @SideOnly(Side.CLIENT)
    private long beamRenderCounter;
    @SideOnly(Side.CLIENT)
    private float beamRenderScale;

    @Override
    public void func_73660_a() {
        if (tick <= 0) {
            if (!canSeeSky() && active) {
                deactivate();
                return;
            }

            effect = HCBeacons.getEffect(field_145850_b, field_174879_c, field_145850_b.func_180495_p(field_174879_c.func_177977_b()));

            tick = DEFAULT_TICK_RATE;

            if (effect != null) {
                BlockStateIngredient structureBlock = effect.getStructureBlock();
                currentLevel = calcLevel(structureBlock);
                if (currentLevel > 0) {
                    if (!active) {
                        activate();
                    }
                    effect.apply(effect.getEntitiesInRange(field_145850_b, field_174879_c, currentLevel), field_145850_b, field_174879_c, currentLevel);
                    calcSegments();
                    tick = effect.getTickRate();
                } else if (active) {
                    deactivate();
                }
            }
        }
        tick--;
    }

    private void activate() {
        CapabilityBeacon storage = field_145850_b.getCapability(CapabilityBeacon.BEACON_CAPABILITY, EnumFacing.UP);
        if (storage != null) {
            storage.addBeacon(field_174879_c, currentLevel);
        }
        effect.onBeaconCreate(field_145850_b, field_174879_c, currentLevel);
        WorldUtils.playBroadcast(field_145850_b, effect.getActivationSound());
        this.field_145850_b.func_72872_a(EntityPlayerMP.class, new AxisAlignedBB(field_174879_c, field_174879_c.func_177982_a(1, -4, 1)).func_72314_b(10.0D, 5.0D, 10.0D)).forEach(player -> CriteriaTriggers.field_192131_k.func_192180_a(player, this));
        this.active = true;
    }

    private void deactivate() {
        this.segments.clear();
        if (effect != null) {
            effect.onBeaconBreak(field_145850_b, field_174879_c, currentLevel);
            WorldUtils.playBroadcast(field_145850_b, effect.getDeactivationSound());
            this.effect = null;
            this.active = false;
        }
        SpawnBeaconEffect.removeAll(func_174877_v());
        CapabilityBeacon storage = field_145850_b.getCapability(CapabilityBeacon.BEACON_CAPABILITY, EnumFacing.UP);
        if (storage != null) {
            storage.removeBeacon(field_174879_c);
        }
        func_70296_d();
    }

    public boolean canSeeSky() {
        BlockPos.MutableBlockPos pos;
        final int height = field_145850_b.field_73011_w.func_177495_o() ? 128 : 256;
        for (pos = new BlockPos.MutableBlockPos(func_174877_v().func_177984_a()); pos.func_177956_o() < height; pos.func_185336_p(pos.func_177956_o() + 1)) {
            IBlockState state = field_145850_b.func_180495_p(pos);
            if (field_145850_b.field_73011_w.func_177495_o() && state.func_177230_c() == Blocks.field_150357_h)
                return true;
            if (state.func_177230_c().getLightOpacity(state, field_145850_b, pos) > 127)
                return false;
        }
        return true;
    }

    @SideOnly(Side.CLIENT)
    public float getBeamScale() {

        int i = (int) (this.field_145850_b.func_82737_E() - this.beamRenderCounter);
        this.beamRenderCounter = this.field_145850_b.func_82737_E();

        if (i > 1) {
            this.beamRenderScale -= (float) i / 40.0F;

            if (this.beamRenderScale < 0.0F) {
                this.beamRenderScale = 0.0F;
            }
        }

        this.beamRenderScale += 0.025F;

        if (this.beamRenderScale > 1.0F) {
            this.beamRenderScale = 1.0F;
        }

        return this.beamRenderScale;
    }

    private void calcSegments() {
        this.segments.clear();
        float[] color = ColorUtils.getColorFromBlock(field_145850_b, func_174877_v().func_177984_a(), func_174877_v());
        if (effect != null) {
            color = effect.getBaseBeaconBeamColor(field_174879_c);
        }

        BeamSegment segment = new BeamSegment(color);
        this.segments.add(segment);

        BlockPos.MutableBlockPos pos;
        for (pos = new BlockPos.MutableBlockPos(func_174877_v().func_177984_a()); pos.func_177956_o() < 256; pos.func_189536_c(EnumFacing.UP)) {
            color = ColorUtils.getColorFromBlock(field_145850_b, pos, func_174877_v());
            if (!Arrays.equals(color, new float[]{1, 1, 1})) {
                color = ColorUtils.average(color, segment.func_177263_b());
                if (Arrays.equals(color, segment.func_177263_b())) {
                    segment.func_177262_a();
                } else {
                    segment = new BeamSegment(color);
                    segments.add(segment);
                }
            } else {
                segment.func_177262_a();
            }
        }
    }

    public List<BeamSegment> getSegments() {
        return segments;
    }


    public int calcLevel(BlockStateIngredient structureBlock) {
        IBlockState stateAtPos;
        int r;
        for (r = 1; r <= 4; r++) {
            for (int x = -r; x <= r; x++) {
                for (int z = -r; z <= r; z++) {
                    stateAtPos = field_145850_b.func_180495_p(field_174879_c.func_177982_a(x, -r, z));
                    if (!structureBlock.test(field_145850_b, field_174879_c, stateAtPos)) {
                        return r - 1;
                    }
                }
            }
        }
        return r - 1;
    }

    @Override
    public NBTTagCompound func_189515_b(NBTTagCompound compound) {
        //TODO - This should all be in a class related to SpawnBeaconEffect not here
        NBTTagCompound tag = new NBTTagCompound();
        NBTUtil.func_190009_a(tag, type);
        compound.func_74782_a("type", tag);

        if (SpawnBeaconEffect.SPAWN_LIST.containsKey(this.func_174877_v())) {
            NBTTagList list = new NBTTagList();
            for (SpawnBeaconEffect.BindingPoint point : SpawnBeaconEffect.SPAWN_LIST.get(this.func_174877_v())) {
                list.func_74742_a(point.serializeNBT());
            }
            compound.func_74782_a("spawns", list);
        }

        return super.func_189515_b(compound);
    }

    @Override
    public void func_145839_a(NBTTagCompound compound) {
        //TODO - This should all be in a class related to SpawnBeaconEffect not here

        NBTTagCompound type = (NBTTagCompound) compound.func_74781_a("type");
        this.type = NBTUtil.func_190008_d(type);
        if (compound.func_74764_b("spawns")) {
            NBTTagList list = compound.func_150295_c("spawns", 10);
            HashSet<SpawnBeaconEffect.BindingPoint> points = Sets.newHashSet();
            for (Iterator<NBTBase> iter = list.iterator(); iter.hasNext(); ) {
                NBTTagCompound tag = (NBTTagCompound) iter.next();
                points.add(new SpawnBeaconEffect.BindingPoint(tag));
            }
            SpawnBeaconEffect.SPAWN_LIST.put(this.func_174877_v(), points);
        }
        super.func_145839_a(compound);
    }

    public boolean processInteraction(World world, EntityPlayer player, ItemStack stack) {
        if (player.func_184812_l_() && !stack.func_190926_b()) {
            if (stack.func_77973_b() instanceof ItemBlock) {
                Block block = ((ItemBlock) stack.func_77973_b()).func_179223_d();
                IBlockState state = block.func_176203_a(stack.func_77960_j());
                if (HCBeacons.isValidBeaconBase(state)) {
                    int r;
                    for (r = 1; r <= 4; r++) {
                        for (int x = -r; x <= r; x++) {
                            for (int z = -r; z <= r; z++) {
                                this.field_145850_b.func_175656_a(func_174877_v().func_177982_a(x, -r, z), state);
                            }
                        }
                    }
                }
            }
        }

        if (!world.field_72995_K) {
            if (effect != null) {
                boolean interacted = this.effect.onPlayerInteracted(world, func_174877_v(), func_191979_s() - 1, player, player.func_184600_cs(), stack);
                if (interacted)
                    this.field_145850_b.func_175669_a(1023, func_174877_v(), 0);
                return interacted;
            }
        }
        return false;
    }

    @Override
    public int func_191979_s() {
        return currentLevel;
    }

    public boolean isEnabled() {
        return active;
    }


    public void onRemoved() {
        deactivate();
    }


    public class BeamSegment extends net.minecraft.tileentity.TileEntityBeacon.BeamSegment {
        public BeamSegment(float[] colorsIn) {
            super(colorsIn);
        }

        @Override
        protected void func_177262_a() {
            super.func_177262_a();
        }
    }
}
