package betterwithmods.module.hardcore.crafting;

import betterwithmods.common.entity.EntityUrn;
import betterwithmods.library.common.event.block.SpawnGolemEvent;
import betterwithmods.library.common.modularity.impl.Feature;
import com.google.common.base.Predicate;
import net.minecraft.advancements.CriteriaTriggers;
import net.minecraft.block.BlockSkull;
import net.minecraft.block.material.Material;
import net.minecraft.block.state.BlockWorldState;
import net.minecraft.block.state.IBlockState;
import net.minecraft.block.state.pattern.BlockMaterialMatcher;
import net.minecraft.block.state.pattern.BlockPattern;
import net.minecraft.block.state.pattern.BlockStateMatcher;
import net.minecraft.block.state.pattern.FactoryBlockPattern;
import net.minecraft.entity.boss.EntityWither;
import net.minecraft.entity.monster.EntityIronGolem;
import net.minecraft.entity.monster.EntitySnowman;
import net.minecraft.entity.player.EntityPlayerMP;
import net.minecraft.init.Blocks;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.tileentity.TileEntitySkull;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.EnumParticleTypes;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.EnumDifficulty;
import net.minecraft.world.World;
import net.minecraftforge.event.entity.EntityJoinWorldEvent;
import net.minecraftforge.fml.common.event.FMLInitializationEvent;
import net.minecraftforge.fml.common.event.FMLPreInitializationEvent;
import net.minecraftforge.fml.common.eventhandler.SubscribeEvent;

import java.util.Random;

public class HCGolems extends Feature {

    private static boolean disableIronGolems;

    @Override
    public String getDescription() {
        return "Change the spawn of Golems and the Wither Boss";
    }


    @SubscribeEvent
    public void onSpawnGolem(SpawnGolemEvent event) {
        event.setCanceled(true);
    }

    @SubscribeEvent
    public void onEntityJoin(EntityJoinWorldEvent event) {
        if (!disableIronGolems)
            return;
        if (event.getEntity() instanceof EntityIronGolem) {
            EntityIronGolem golem = (EntityIronGolem) event.getEntity();
            if (!golem.func_70850_q()) {
                event.setCanceled(true);
            }
        }
    }

    @Override
    public boolean hasEvent() {
        return true;
    }

    @Override
    public void onPreInit(FMLPreInitializationEvent event) {
        disableIronGolems = loadProperty("Disable Village Iron Golem Spawns", true).setComment("WARNING: Stops all non-player created Iron Golem Spawns").get();
    }

    @Override
    public void onInit(FMLInitializationEvent event) {
        BlockPattern wither = FactoryBlockPattern.func_177660_a().func_177659_a("^^^", "###", "~#~").func_177662_a('#', BlockWorldState.func_177510_a(BlockStateMatcher.func_177638_a(Blocks.field_150425_aM))).func_177662_a('^', IS_WITHER_SKELETON).func_177662_a('~', BlockWorldState.func_177510_a(BlockMaterialMatcher.func_189886_a(Material.field_151579_a))).func_177661_b();
        BlockPattern snowGolem = FactoryBlockPattern.func_177660_a().func_177659_a("^", "#", "#").func_177662_a('^', BlockWorldState.func_177510_a(IS_PUMPKIN)).func_177662_a('#', BlockWorldState.func_177510_a(BlockStateMatcher.func_177638_a(Blocks.field_150433_aE))).func_177661_b();
        BlockPattern ironGolem = FactoryBlockPattern.func_177660_a().func_177659_a("~^~", "###", "~#~").func_177662_a('^', BlockWorldState.func_177510_a(IS_PUMPKIN)).func_177662_a('#', BlockWorldState.func_177510_a(BlockStateMatcher.func_177638_a(Blocks.field_150339_S))).func_177662_a('~', BlockWorldState.func_177510_a(BlockMaterialMatcher.func_189886_a(Material.field_151579_a))).func_177661_b();

        EntityUrn.addHandler((world, pos, random) -> formWitherBoss(world, pos, random, wither));
        EntityUrn.addHandler((world, pos, random) -> formGolem(world, pos, random, snowGolem, HCGolems::spawnSnowGolem));
        EntityUrn.addHandler((world, pos, random) -> formGolem(world, pos, random, ironGolem, HCGolems::spawnIronGolem));
    }


    public static boolean formWitherBoss(World world, BlockPos pos, Random random, BlockPattern pattern) {
        TileEntity tile = world.func_175625_s(pos);

        if (tile instanceof TileEntitySkull) {
            if (((TileEntitySkull) tile).func_145904_a() == 1 && pos.func_177956_o() >= 2 && world.func_175659_aa() != EnumDifficulty.PEACEFUL && !world.field_72995_K) {
                formGolem(world, pos, random, pattern, HCGolems::spawnWither);
                return true;
            }
        }
        return false;
    }

    public static boolean formGolem(World world, BlockPos pos, Random random, BlockPattern pattern, GolemSpawner spawner) {
        BlockPattern.PatternHelper match = pattern.func_177681_a(world, pos);
        if (match != null) {
            spawner.onMatch(world, pos, random, pattern, match);
            return true;
        }
        return false;
    }

    @FunctionalInterface
    private interface GolemSpawner {
        void onMatch(World world, BlockPos pos, Random random, BlockPattern pattern, BlockPattern.PatternHelper helper);
    }


    private static final Predicate<IBlockState> IS_PUMPKIN = state -> state != null && (state.func_177230_c() == Blocks.field_150423_aK || state.func_177230_c() == Blocks.field_150428_aP);
    private static final Predicate<BlockWorldState> IS_WITHER_SKELETON = state -> state.func_177509_a() != null && state.func_177509_a().func_177230_c() == Blocks.field_150465_bP && state.func_177507_b() instanceof TileEntitySkull && ((TileEntitySkull) state.func_177507_b()).func_145904_a() == 1;


    private static void spawnSnowGolem(World world, BlockPos pos, Random random, BlockPattern pattern, BlockPattern.PatternHelper match) {
        for (int i = 0; i < pattern.func_177685_b(); ++i) {
            BlockWorldState blockworldstate = match.func_177670_a(0, i, 0);
            world.func_180501_a(blockworldstate.func_177508_d(), Blocks.field_150350_a.func_176223_P(), 2);
        }

        EntitySnowman entitysnowman = new EntitySnowman(world);
        BlockPos blockpos1 = match.func_177670_a(0, 2, 0).func_177508_d();
        entitysnowman.func_70012_b((double) blockpos1.func_177958_n() + 0.5D, (double) blockpos1.func_177956_o() + 0.05D, (double) blockpos1.func_177952_p() + 0.5D, 0.0F, 0.0F);
        world.func_72838_d(entitysnowman);

        for (EntityPlayerMP entityplayermp : world.func_72872_a(EntityPlayerMP.class, entitysnowman.func_174813_aQ().func_186662_g(5.0D))) {
            CriteriaTriggers.field_192133_m.func_192229_a(entityplayermp, entitysnowman);
        }

        for (int l = 0; l < 120; ++l) {
            world.func_175688_a(EnumParticleTypes.SNOW_SHOVEL, (double) blockpos1.func_177958_n() + world.field_73012_v.nextDouble(), (double) blockpos1.func_177956_o() + world.field_73012_v.nextDouble() * 2.5D, (double) blockpos1.func_177952_p() + world.field_73012_v.nextDouble(), 0.0D, 0.0D, 0.0D);
        }

        for (int i1 = 0; i1 < pattern.func_177685_b(); ++i1) {
            BlockWorldState blockworldstate2 = match.func_177670_a(0, i1, 0);
            world.func_175722_b(blockworldstate2.func_177508_d(), Blocks.field_150350_a, false);
        }
    }


    private static void spawnIronGolem(World world, BlockPos pos, Random random, BlockPattern pattern, BlockPattern.PatternHelper match) {

        for (int j = 0; j < pattern.func_177684_c(); ++j) {
            for (int k = 0; k < pattern.func_177685_b(); ++k) {
                world.func_180501_a(match.func_177670_a(j, k, 0).func_177508_d(), Blocks.field_150350_a.func_176223_P(), 2);
            }
        }

        BlockPos blockpos = match.func_177670_a(1, 2, 0).func_177508_d();
        EntityIronGolem entityirongolem = new EntityIronGolem(world);
        entityirongolem.func_70849_f(true);
        entityirongolem.func_70012_b((double) blockpos.func_177958_n() + 0.5D, (double) blockpos.func_177956_o() + 0.05D, (double) blockpos.func_177952_p() + 0.5D, 0.0F, 0.0F);
        world.func_72838_d(entityirongolem);

        for (EntityPlayerMP entityplayermp1 : world.func_72872_a(EntityPlayerMP.class, entityirongolem.func_174813_aQ().func_186662_g(5.0D))) {
            CriteriaTriggers.field_192133_m.func_192229_a(entityplayermp1, entityirongolem);
        }

        for (int j1 = 0; j1 < 120; ++j1) {
            world.func_175688_a(EnumParticleTypes.SNOWBALL, (double) blockpos.func_177958_n() + world.field_73012_v.nextDouble(), (double) blockpos.func_177956_o() + world.field_73012_v.nextDouble() * 3.9D, (double) blockpos.func_177952_p() + world.field_73012_v.nextDouble(), 0.0D, 0.0D, 0.0D);
        }

        for (int k1 = 0; k1 < pattern.func_177684_c(); ++k1) {
            for (int l1 = 0; l1 < pattern.func_177685_b(); ++l1) {
                BlockWorldState blockworldstate1 = match.func_177670_a(k1, l1, 0);
                world.func_175722_b(blockworldstate1.func_177508_d(), Blocks.field_150350_a, false);
            }
        }

    }


    private static void spawnWither(World world, BlockPos pos, Random random, BlockPattern pattern, BlockPattern.PatternHelper match) {
        for (int i = 0; i < 3; ++i) {
            BlockWorldState blockworldstate = match.func_177670_a(i, 0, 0);
            world.func_180501_a(blockworldstate.func_177508_d(), blockworldstate.func_177509_a().func_177226_a(BlockSkull.field_176417_b, Boolean.valueOf(true)), 2);
        }

        for (int j = 0; j < pattern.func_177684_c(); ++j) {
            for (int k = 0; k < pattern.func_177685_b(); ++k) {
                BlockWorldState blockworldstate1 = match.func_177670_a(j, k, 0);
                world.func_180501_a(blockworldstate1.func_177508_d(), Blocks.field_150350_a.func_176223_P(), 2);
            }
        }

        BlockPos blockpos = match.func_177670_a(1, 0, 0).func_177508_d();
        EntityWither entitywither = new EntityWither(world);
        BlockPos blockpos1 = match.func_177670_a(1, 2, 0).func_177508_d();
        entitywither.func_70012_b((double) blockpos1.func_177958_n() + 0.5D, (double) blockpos1.func_177956_o() + 0.55D, (double) blockpos1.func_177952_p() + 0.5D, match.func_177669_b().func_176740_k() == EnumFacing.Axis.X ? 0.0F : 90.0F, 0.0F);
        entitywither.field_70761_aq = match.func_177669_b().func_176740_k() == EnumFacing.Axis.X ? 0.0F : 90.0F;
        entitywither.func_82206_m();

        for (EntityPlayerMP entityplayermp : world.func_72872_a(EntityPlayerMP.class, entitywither.func_174813_aQ().func_186662_g(50.0D))) {
            CriteriaTriggers.field_192133_m.func_192229_a(entityplayermp, entitywither);
        }

        world.func_72838_d(entitywither);

        for (int l = 0; l < 120; ++l) {
            world.func_175688_a(EnumParticleTypes.SNOWBALL, (double) blockpos.func_177958_n() + world.field_73012_v.nextDouble(), (double) (blockpos.func_177956_o() - 2) + world.field_73012_v.nextDouble() * 3.9D, (double) blockpos.func_177952_p() + world.field_73012_v.nextDouble(), 0.0D, 0.0D, 0.0D);
        }

        for (int i1 = 0; i1 < pattern.func_177684_c(); ++i1) {
            for (int j1 = 0; j1 < pattern.func_177685_b(); ++j1) {
                BlockWorldState blockworldstate2 = match.func_177670_a(i1, j1, 0);
                world.func_175722_b(blockworldstate2.func_177508_d(), Blocks.field_150350_a, false);
            }
        }
    }
}

