package betterwithmods.module.hardcore.world;

import betterwithmods.library.common.modularity.impl.Feature;
import betterwithmods.util.PlayerUtils;
import net.minecraft.entity.Entity;
import net.minecraft.entity.EntityList;
import net.minecraft.entity.item.EntityBoat;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.ItemBanner;
import net.minecraft.item.ItemStack;
import net.minecraft.item.crafting.Ingredient;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.util.ResourceLocation;
import net.minecraftforge.fml.common.Loader;
import net.minecraftforge.fml.common.event.FMLPostInitializationEvent;
import net.minecraftforge.fml.common.event.FMLPreInitializationEvent;
import net.minecraftforge.fml.common.eventhandler.SubscribeEvent;
import net.minecraftforge.fml.common.gameevent.TickEvent;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;


public class HCBoating extends Feature {

    //Quark Boat Sail Compat
    private static final String TAG_BANNER = "quark:banner";
    public static HashMap<Ingredient, Integer> SPEED_ITEMS;
    public static List<ResourceLocation> BOAT_ENTRIES;
    public static int defaultSpeed;

    @SubscribeEvent
    public void onTick(TickEvent.PlayerTickEvent event) {
        if (event.phase == TickEvent.Phase.START)
            return;
        if (!event.player.field_70170_p.field_72995_K)
            return;
        EntityPlayer player = event.player;
        Entity riding = player.func_184187_bx();
        if (riding != null && BOAT_ENTRIES.stream().anyMatch(r -> EntityList.func_180123_a(riding, r))) {
            Set<ItemStack> stacks = PlayerUtils.getHolding(player);
            int speed = defaultSpeed;
            for (ItemStack stack : stacks) {
                if (speed <= defaultSpeed) {
                    if (!stack.func_190926_b()) {
                        speed = SPEED_ITEMS.entrySet().stream().filter(e -> e.getKey().apply(stack)).mapToInt(Map.Entry::getValue).findAny().orElse(defaultSpeed);
                    }
                }
            }

            if (Loader.isModLoaded("quark")) {
                int quarkCompat = quarkCompatSpeed((EntityBoat) riding);
                if (quarkCompat > 0)
                    speed = quarkCompat;
            }

            riding.field_70159_w *= (speed / 100f);
            riding.field_70179_y *= (speed / 100f);
        }
    }

    private static int quarkCompatSpeed(EntityBoat boat) {
        NBTTagCompound tag = boat.getEntityData();
        if (tag.func_74764_b(TAG_BANNER)) {
            NBTTagCompound cmp = boat.getEntityData().func_74775_l(TAG_BANNER);
            ItemStack stack = new ItemStack(cmp);
            if (!stack.func_190926_b() && stack.func_77973_b() instanceof ItemBanner) {
                return SPEED_ITEMS.entrySet().stream().filter(e -> e.getKey().apply(stack)).findFirst().map(Map.Entry::getValue).orElse(0);
            }
        }
        return 0;
    }

    @Override
    public String getDescription() {
        return "Boats are much slower as simple oars are not very good for speed. To go faster you must hold a Wind Sail.";
    }

    @Override
    public void onPreInitClient(FMLPreInitializationEvent event) {
        config().loadRecipeCondition("boatshovel", getCategory(), "Boat Requires Oar", "Make boat recipe require a wooden shovel for the oars", true);
    }

    @Override
    public void onPostInit(FMLPostInitializationEvent event) {
        SPEED_ITEMS = config().loadItemStackIntMap("Speed Items", getCategory(), "Items which speed up a boat when held, value is a percentage of the vanilla speed", new String[]{
                "betterwithmods:wind_sail=100",
                "minecraft:banner:*=100"
        });
        defaultSpeed = loadProperty("Default Speed modifier", 50).setComment("Speed modifier when not holding any sail type item").get();
        BOAT_ENTRIES = config().loadResouceLocations("Boat List", getCategory(), "Registry name for entities which are considered boats", new String[]{"minecraft:boat"});
    }

    @Override
    public boolean hasEvent() {
        return true;
    }

}
