package betterwithmods.module.hardcore.world;

import betterwithmods.common.BWMBlocks;
import betterwithmods.common.BWMItems;
import betterwithmods.common.blocks.BlockPlanter;
import betterwithmods.library.common.event.block.EntityCollidedEvent;
import betterwithmods.library.common.modularity.impl.Feature;
import betterwithmods.library.common.recipes.RecipeMatchers;
import betterwithmods.library.common.recipes.RecipeRemover;
import betterwithmods.library.utils.ingredient.StackIngredient;
import betterwithmods.library.utils.ingredient.collections.IngredientSet;
import betterwithmods.module.internal.RecipeRegistry;
import betterwithmods.util.PlayerUtils;
import net.minecraft.block.Block;
import net.minecraft.block.BlockFarmland;
import net.minecraft.block.BlockGrass;
import net.minecraft.block.IGrowable;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.item.EntityItem;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.init.Blocks;
import net.minecraft.init.Items;
import net.minecraft.item.EnumDyeColor;
import net.minecraft.item.ItemStack;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.World;
import net.minecraftforge.common.IPlantable;
import net.minecraftforge.event.entity.player.BonemealEvent;
import net.minecraftforge.event.entity.player.PlayerInteractEvent;
import net.minecraftforge.fml.common.event.FMLInitializationEvent;
import net.minecraftforge.fml.common.event.FMLPreInitializationEvent;
import net.minecraftforge.fml.common.eventhandler.SubscribeEvent;

import javax.annotation.Nullable;

/**
 * Created by primetoxinz on 5/14/17.
 */


public class HCBonemeal extends Feature {
    private static boolean removeBonemealRecipe;
    public static IngredientSet FERTILIZERS = new IngredientSet();

    public static void registerFertilizer(ItemStack stack) {
        FERTILIZERS.add(StackIngredient.fromStacks(stack));
    }

    @SubscribeEvent
    public void onBonemeal(BonemealEvent e) {
        if (!PlayerUtils.isSurvival(e.getEntityPlayer()))
            return;
        if (!(e.getBlock().func_177230_c() instanceof BlockGrass) && !(e.getBlock().func_177230_c() instanceof BlockPlanter) && e.getBlock().func_177230_c() instanceof IGrowable) {
            IBlockState below = e.getWorld().func_180495_p(e.getPos().func_177977_b());
            below.func_177230_c().func_180649_a(e.getWorld(), e.getPos().func_177977_b(), e.getEntityPlayer());
            e.setCanceled(true);
        }
    }

    @SubscribeEvent
    public void onEntityCollideWith(EntityCollidedEvent event) {
        if (event.getWorld().field_72995_K)
            return;
        if (event.getState().func_177230_c() instanceof BlockFarmland || event.getState().func_177230_c() == BlockPlanter.getBlock(BlockPlanter.Type.FARMLAND)) {
            if (event.getEntity() instanceof EntityItem) {
                ItemStack stack = ((EntityItem) event.getEntity()).func_92059_d();
                if (FERTILIZERS.test(stack)) {
                    fertilizerBlock(event.getWorld(), event.getPos(), stack, null);
                }
            }
        }
    }

    @SubscribeEvent
    public void onItemUse(PlayerInteractEvent.RightClickBlock e) {
        ItemStack stack = e.getItemStack();
        if (!FERTILIZERS.test(stack))
            return;
        World world = e.getWorld();
        BlockPos pos = e.getPos();
        fertilizerBlock(world, pos, stack, e.getEntityPlayer());
    }

    private static void fertilizerBlock(World world, BlockPos pos, ItemStack stack, @Nullable EntityPlayer player) {
        IBlockState state = world.func_180495_p(pos);
        Block block = state.func_177230_c();
        if (block instanceof IPlantable) {
            IBlockState below = world.func_180495_p(pos.func_177977_b());
            if (processFertilization(below, world, pos.func_177977_b())) {
                if (player == null || !player.field_71075_bZ.field_75098_d)
                    stack.func_190918_g(1);
            }
        } else if (processFertilization(state, world, pos)) {
            if (player == null || !player.field_71075_bZ.field_75098_d)
                stack.func_190918_g(1);
        }
    }

    private static boolean processFertilization(IBlockState state, World world, BlockPos pos) {
        Block block = state.func_177230_c();
        if (block == Blocks.field_150458_ak) {
            world.func_175656_a(pos, BWMBlocks.FERTILE_FARMLAND.func_176223_P().func_177226_a(BlockFarmland.field_176531_a, world.func_180495_p(pos).func_177229_b(BlockFarmland.field_176531_a)));
            world.func_175718_b(2005, pos.func_177984_a(), 0);
            return true;
        } else if (block == BlockPlanter.getBlock(BlockPlanter.Type.FARMLAND)) {
            world.func_175656_a(pos, BlockPlanter.getBlock(BlockPlanter.Type.FERTILE).func_176223_P());
            world.func_175718_b(2005, pos.func_177984_a(), 0);
            return true;
        }
        return false;
    }

    @Override
    public String getDescription() {
        return "Removes the ability to instant-grow crops and trees with bonemeal";
    }

    @Override
    public void onPreInit(FMLPreInitializationEvent event) {
        removeBonemealRecipe = loadProperty("Remove Bonemeal Crafting Recipe", true).get();
    }

    @Override
    public void registerRecipes() {
        if (removeBonemealRecipe) {
            RecipeRegistry.removeRecipe(new RecipeRemover<>(RecipeMatchers.OUTPUT_SIZE, new ItemStack(Items.field_151100_aR, 3, EnumDyeColor.WHITE.func_176767_b())));
        }
    }

    @Override
    public void onInit(FMLInitializationEvent event) {
        registerFertilizer(new ItemStack(Items.field_151100_aR, 1, EnumDyeColor.WHITE.func_176767_b()));
        registerFertilizer(new ItemStack(BWMItems.FERTILIZER));
    }

    @Override
    public boolean hasEvent() {
        return true;
    }
}
