package betterwithmods.module.hardcore.world.structures;

import betterwithmods.common.BWMBlocks;
import betterwithmods.common.blocks.BlockAesthetic;
import betterwithmods.library.common.event.structure.StructureSetBlockEvent;
import betterwithmods.library.common.modularity.impl.Feature;
import betterwithmods.library.common.recipes.RecipeMatchers;
import betterwithmods.library.common.recipes.RecipeRemover;
import betterwithmods.library.utils.ingredient.blockstate.BlockIngredient;
import betterwithmods.module.internal.RecipeRegistry;
import com.google.common.collect.Sets;
import net.minecraft.block.BlockLadder;
import net.minecraft.block.BlockPlanks;
import net.minecraft.init.Blocks;
import net.minecraft.init.Items;
import net.minecraft.item.ItemStack;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.math.AxisAlignedBB;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.gen.structure.ComponentScatteredFeaturePieces;
import net.minecraftforge.fml.common.event.FMLInitializationEvent;
import net.minecraftforge.fml.common.event.FMLPreInitializationEvent;
import net.minecraftforge.fml.common.eventhandler.SubscribeEvent;

import java.util.Set;

/**
 * Created by primetoxinz on 4/20/17.
 */


public class HCStructures extends Feature {
    public static int HARDCORE_STRUCTURE_RADIUS;
    public static Set<StructureChanger> DESERT_TEMPLE = Sets.newHashSet();
    public static StructureChanger ABANDONED_DESERT_TEMPLE, NORMAL_DESERT_TEMPLE;
    private static Set<StructureChanger> JUNGLE_TEMPLE = Sets.newHashSet();
    private static StructureChanger ABANDONED_JUNGLE_TEMPLE, NORMAL_JUNGLE_TEMPLE;
    private static Set<StructureChanger> WITCH_HUT = Sets.newHashSet();
    private static StructureChanger WITCH_HUT_CHANGER;
    private boolean disableRecipes;

    @Override
    public String getDescription() {
        return "Makes it so structures are looted within a radius of spawn and unlooted outside of that radius. Encourages exploration and also makes unlooted structures the only source of Enchanting Tables and Brewing Stands.";
    }


    @Override
    public void onPreInit(FMLPreInitializationEvent event) {
        HARDCORE_STRUCTURE_RADIUS = loadProperty("Hardcore EnumStructure Radius", 2000).setComment("Radius from original spawn which structures will be abandoned in").get();
        disableRecipes = loadProperty("Disable Recipes", true).setComment("Disable Recipes for blocks that generate only in structures, including Enchanting Tables and Brewing Stands").get();
        ABANDONED_DESERT_TEMPLE = StructureChanger.create(DESERT_TEMPLE, (w, p) -> p.func_177951_i(w.func_175694_M()) < HARDCORE_STRUCTURE_RADIUS * HARDCORE_STRUCTURE_RADIUS);
        NORMAL_DESERT_TEMPLE = StructureChanger.create(DESERT_TEMPLE, (w, p) -> p.func_177951_i(w.func_175694_M()) >= HARDCORE_STRUCTURE_RADIUS * HARDCORE_STRUCTURE_RADIUS);

        ABANDONED_JUNGLE_TEMPLE = StructureChanger.create(JUNGLE_TEMPLE, (w, p) -> p.func_177951_i(w.func_175694_M()) < HARDCORE_STRUCTURE_RADIUS * HARDCORE_STRUCTURE_RADIUS);
        NORMAL_JUNGLE_TEMPLE = StructureChanger.create(JUNGLE_TEMPLE, (w, p) -> p.func_177951_i(w.func_175694_M()) >= HARDCORE_STRUCTURE_RADIUS * HARDCORE_STRUCTURE_RADIUS);

        WITCH_HUT_CHANGER = StructureChanger.create(WITCH_HUT, (w, p) -> true);
    }

    @Override
    public void registerRecipes() {
        if (disableRecipes) {
            RecipeRegistry.removeRecipe(new RecipeRemover<>(RecipeMatchers.OUTPUT, new ItemStack(Blocks.field_150381_bn)));
            RecipeRegistry.removeRecipe(new RecipeRemover<>(RecipeMatchers.OUTPUT, new ItemStack(Items.field_151067_bt)));
        }
    }

    @Override
    public void onInit(FMLInitializationEvent event) {
        ABANDONED_DESERT_TEMPLE
                //Dig hole
                .addChanger(new RelativePosChanger(Blocks.field_150350_a.func_176223_P(),
                        new BlockPos(10, 0, 10),
                        new BlockPos(11, 0, 10),
                        new BlockPos(10, 0, 9)))
                //Remove TNT
                .addChanger(new RelativePosChanger(Blocks.field_150322_A.func_176223_P(), new AxisAlignedBB(9, -13, 9, 11, -13, 11)))
                //Remove pressure plate
                .addChanger(new RelativePosChanger(Blocks.field_150350_a.func_176223_P(), new BlockPos(10, -11, 10)))
                //Place Ladder
                .addChanger(new RelativePosChanger(Blocks.field_150468_ap.func_176223_P().func_177226_a(BlockLadder.field_176382_a, EnumFacing.WEST), new AxisAlignedBB(11, 0, 9, 11, -13, 9)))
                //Deletes loot table
                .addChanger(new ChestLootChanger(null))
                //Replace clay with obsidian
                .addChanger(new IngredientChanger(new BlockIngredient(Blocks.field_150406_ce), Blocks.field_150343_Z.func_176223_P()));

        NORMAL_DESERT_TEMPLE
                .addChanger(new RelativePosChanger(Blocks.field_150381_bn.func_176223_P(), new BlockPos(10, 1, 10)))
                //Replace clay with obsidian
                .addChanger(new IngredientChanger(new BlockIngredient(Blocks.field_150406_ce), Blocks.field_150343_Z.func_176223_P()))
                .addChanger(new IngredientChanger(new BlockIngredient(Blocks.field_150456_au), Blocks.field_150452_aw.func_176223_P()));

        ABANDONED_JUNGLE_TEMPLE
                .addChanger(new IngredientChanger(new BlockIngredient(Blocks.field_150488_af), Blocks.field_150350_a.func_176223_P()))
                .addChanger(new IngredientChanger(new BlockIngredient(Blocks.field_150320_F), Blocks.field_150350_a.func_176223_P()))
                .addChanger(new IngredientChanger(new BlockIngredient(Blocks.field_150367_z), Blocks.field_150350_a.func_176223_P()))
                .addChanger(new IngredientChanger(new BlockIngredient(Blocks.field_150473_bD), Blocks.field_150350_a.func_176223_P()))
                .addChanger(new IngredientChanger(new BlockIngredient(Blocks.field_150479_bC), Blocks.field_150350_a.func_176223_P()))
                .addChanger(new IngredientChanger(new BlockIngredient(Blocks.field_150413_aR), Blocks.field_150350_a.func_176223_P()))
                .addChanger(new IngredientChanger(new BlockIngredient(Blocks.field_150416_aS), Blocks.field_150350_a.func_176223_P()))
                .addChanger(new IngredientChanger(new BlockIngredient(Blocks.field_150442_at), Blocks.field_150350_a.func_176223_P()))
                .addChanger(new RelativePosChanger(Blocks.field_150350_a.func_176223_P(), new BlockPos(5, 3, 10)))
                .addChanger(new RelativePosChanger(Blocks.field_150350_a.func_176223_P(), new BlockPos(6, 3, 10)))
                .addChanger(new ChestLootChanger(null))
                ;


        NORMAL_JUNGLE_TEMPLE
                .addChanger(new RelativePosChanger(BWMBlocks.HAND_CRANK.func_176223_P(), new BlockPos(5, 3, 10)))
                .addChanger(new RelativePosChanger(BWMBlocks.DRAGON_VESSEL.func_176223_P(), new BlockPos(6, 3, 10)))
                .addChanger(new RelativePosChanger(BlockAesthetic.getVariant(BlockAesthetic.Type.CHOPBLOCKBLOOD), new BlockPos(5, 4, 11),new BlockPos(6, 4, 11)));


        WITCH_HUT_CHANGER
                .addChanger(new IngredientChanger(new BlockIngredient(Blocks.field_150457_bL),Blocks.field_150350_a.func_176223_P()))
                .addChanger(new IngredientChanger(new BlockIngredient(Blocks.field_150462_ai),Blocks.field_150382_bo.func_176223_P()));
    }


    @SubscribeEvent
    public void onStructureSetBlock(StructureSetBlockEvent event) {
        if (event.getComponent() instanceof ComponentScatteredFeaturePieces.DesertPyramid) {
            StructureChanger.convert(DESERT_TEMPLE, event);
        } else if (event.getComponent() instanceof ComponentScatteredFeaturePieces.JunglePyramid) {
            StructureChanger.convert(JUNGLE_TEMPLE, event);
        } else if (event.getComponent() instanceof ComponentScatteredFeaturePieces.SwampHut) {
            StructureChanger.convert(WITCH_HUT, event);
        }
    }

    @Override
    public boolean hasTerrainGen() {
        return true;
    }

    @Override
    public boolean hasEvent() {
        return true;
    }
}
