package betterwithmods.module.hardcore.world.structures;

import betterwithmods.library.common.event.structure.StructureLootEvent;
import betterwithmods.library.common.event.structure.StructureSetBlockEvent;
import betterwithmods.library.common.modularity.impl.Feature;
import betterwithmods.library.utils.ingredient.blockstate.BlockIngredient;
import betterwithmods.library.utils.ingredient.blockstate.BlockStateIngredient;
import betterwithmods.library.utils.ingredient.blockstate.MaterialIngredient;
import betterwithmods.library.utils.ingredient.blockstate.PredicateBlockStateIngredient;
import com.google.common.collect.Lists;
import com.google.common.collect.Sets;
import net.minecraft.block.BlockDirt;
import net.minecraft.block.BlockDoor;
import net.minecraft.block.material.Material;
import net.minecraft.entity.monster.EntityIronGolem;
import net.minecraft.init.Blocks;
import net.minecraft.item.EnumDyeColor;
import net.minecraft.item.ItemStack;
import net.minecraft.world.gen.structure.StructureVillagePieces;
import net.minecraftforge.event.entity.EntityJoinWorldEvent;
import net.minecraftforge.fml.common.event.FMLInitializationEvent;
import net.minecraftforge.fml.common.eventhandler.SubscribeEvent;

import java.util.Set;

/**
 * Created by primetoxinz on 5/21/17.
 */


public class HCVillages extends Feature {

    private static Set<StructureChanger> VILLAGE = Sets.newHashSet();

    public static boolean disableVillagerSpawning;
    private int normalRadius, semiabandonedRadius;


    @SubscribeEvent
    public void onStructureSetBlock(StructureSetBlockEvent event) {
        if (event.getComponent() instanceof StructureVillagePieces.Village) {
            StructureChanger.convert(VILLAGE, event);
        }
    }

    @SubscribeEvent
    public void onStructureLoot(StructureLootEvent event) {
        if (event.getComponent() instanceof StructureVillagePieces.Village) {
            StructureChanger.convert(VILLAGE, event);
        }
    }

    @Override
    public String getDescription() {
        return "Makes it so villages with in the reaches of the spawn zone are abandoned and gradually gain more resources the further out. What this means to be gained by the player.";
    }


    @Override
    public void onInit(FMLInitializationEvent event) {
        semiabandonedRadius = loadProperty("Semi-Abandoned Village Radius", 2000).setComment("Block radius from 0,0 at which villages are now semi-abandoned, all villages inside this radius are abandoned").get();
        normalRadius = loadProperty("Normal Village Radius", 3000).setComment("Block radius from 0,0 at which villages are now normal, all villages in between this and semi-abandoned are semi-abandoned").get();

        disableVillagerSpawning = loadProperty("Replace Villager Spawning with Nitwits", true).setComment("Replaces all villager spawns with Nitwits, which have no trades").get();

        StructureChanger ABANDONED = StructureChanger.create(VILLAGE, (w, p) -> p.func_177951_i(w.func_175694_M()) < semiabandonedRadius * semiabandonedRadius);
        StructureChanger SEMIABANDONED = StructureChanger.create(VILLAGE, (w, p) -> p.func_177951_i(w.func_175694_M()) < normalRadius * normalRadius);
        StructureChanger NORMAL = StructureChanger.create(VILLAGE, (w, p) -> p.func_177951_i(w.func_175694_M()) >= normalRadius * normalRadius);

        TableChanger tableChanger = new TableChanger();

        ABANDONED
                .addChanger(tableChanger)
                .addChanger(new BiomeIngredientChanger(new BlockStateIngredient(new ItemStack(Blocks.field_150325_L, 1, EnumDyeColor.BLACK.func_176765_a())), Blocks.field_150344_f.func_176223_P()))
                .addChanger(new IngredientChanger(new BlockIngredient(Blocks.field_150462_ai), Blocks.field_150350_a.func_176223_P()))
                .addChanger(new IngredientChanger(new BlockIngredient(Blocks.field_150342_X), Blocks.field_150350_a.func_176223_P()))
                .addChanger(new IngredientChanger(new BlockIngredient(Blocks.field_150478_aa), Blocks.field_150350_a.func_176223_P()))
                .addChanger(new IngredientChanger(new BlockIngredient(Blocks.field_150460_al), Blocks.field_150347_e.func_176223_P()))
                .addChanger(new IngredientChanger(new BlockIngredient(Blocks.field_150458_ak), Blocks.field_150346_d.func_176223_P().func_177226_a(BlockDirt.field_176386_a, BlockDirt.DirtType.COARSE_DIRT)))
                .addChanger(new IngredientChanger(new MaterialIngredient(Material.field_151592_s), Blocks.field_150350_a.func_176223_P()))
                .addChanger(new IngredientChanger(new MaterialIngredient(Material.field_151585_k), Blocks.field_150350_a.func_176223_P()))
                .addChanger(new IngredientChanger(new MaterialIngredient(Material.field_151586_h), Blocks.field_150346_d.func_176223_P()))
                .addChanger(new IngredientChanger(new PredicateBlockStateIngredient((world, pos) -> world.func_180495_p(pos).func_177230_c() instanceof BlockDoor), Blocks.field_150350_a.func_176223_P()))
                .addChanger(new ChestLootChanger(null));


        SEMIABANDONED
                .addChanger(tableChanger)
                .addChanger(new BiomeIngredientChanger(new BlockStateIngredient(new ItemStack(Blocks.field_150325_L, 1, EnumDyeColor.BLACK.func_176765_a())), Blocks.field_150344_f.func_176223_P()))
                .addChanger(new IngredientChanger(new BlockIngredient(Blocks.field_150462_ai), Blocks.field_150350_a.func_176223_P()))
                .addChanger(new IngredientChanger(new BlockIngredient(Blocks.field_150342_X), Blocks.field_150350_a.func_176223_P()))
                .addChanger(new IngredientChanger(new BlockIngredient(Blocks.field_150478_aa), Blocks.field_150350_a.func_176223_P()))
                .addChanger(new IngredientChanger(new BlockIngredient(Blocks.field_150460_al), Blocks.field_150350_a.func_176223_P()))
                .addChanger(new IngredientChanger(new BlockIngredient(Blocks.field_150349_c), Blocks.field_150346_d.func_176223_P()))
                .addChanger(new IngredientChanger(new MaterialIngredient(Material.field_151592_s), Blocks.field_150350_a.func_176223_P()))
                .addChanger(new IngredientChanger(new MaterialIngredient(Material.field_151586_h), Blocks.field_150346_d.func_176223_P()))
                .addChanger(new IngredientChanger(new PredicateBlockStateIngredient((world, pos) -> world.func_180495_p(pos).func_177230_c() instanceof BlockDoor), Blocks.field_150350_a.func_176223_P()))
                .addChanger(new ChestLootChanger(null))
                .addChanger(new CropTypeChanger(Lists.newArrayList(Blocks.field_150469_bN, Blocks.field_150459_bM, Blocks.field_150350_a)));
        //TODO configs for crop lists
        NORMAL
                .addChanger(tableChanger)
                .addChanger(new CropTypeChanger(Lists.newArrayList(Blocks.field_150464_aj, Blocks.field_185773_cZ, Blocks.field_150469_bN, Blocks.field_150459_bM)));

    }

    @Override
    public boolean hasEvent() {
        return true;
    }

    @Override
    public boolean hasTerrainGen() {
        return true;
    }
}



